/*
    DABlin - capital DAB experience
    Copyright (C) 2015-2019 Stefan Pöschel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// --- BitWriter ------------------------------------------------------------

#include	<stdlib.h>
#include	<stdint.h>
#include	<vector>

class BitWriter {
private:
        std::vector<uint8_t> data;
        size_t byte_bits;
public:
        BitWriter() {Reset();}

        void Reset();
        void AddBits(int data_new, size_t count);
        void AddBytes(const uint8_t *data, size_t len);
        const std::vector<uint8_t> GetData() {
	   return data;
	}

        void WriteAudioMuxLengthBytes();        // needed for LATM
};

