#
/*
 *    Copyright (C) 2014 .. 2020
 *    Jan van Katwijk (J.vanKatwijk@gmail.com)
 *    Lazy Chair Computing
 *
 *    This file is part of the Qt-DAB program
 *
 *    Qt-DAB is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    Qt-DAB is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Qt-DAB if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include	"dab-tables.h"
#include	"data-descriptor.h"


	dataDescriptor::dataDescriptor (packetdata *pd):
	                                       myFrame (nullptr) {
        setupUi (&myFrame);
        myFrame. show();
	serviceName  -> setText (pd -> serviceName);
	serviceLabel    -> setText
                      (QString().number (pd -> SId, 16). toUpper());
        QFont font      = serviceLabel -> font();
        font. setBold (true);
	subChannelId -> setText (QString::number (pd -> subchId));
	startAddress -> setText (QString::number (pd -> startAddr));
	Length       -> setText (QString::number (pd -> length));
	bitrate      -> setText (QString::number (pd -> bitRate));
	packetAddress-> setText (QString::number (pd -> packetAddress));
	QString protL	= getProtectionLevel (pd -> shortForm,
	                                      pd -> protLevel);
	protectionLevel   -> setText (protL);
	appType	     -> setText (getDSCTy (pd -> DSCTy));
	FECscheme    -> setText (getFECscheme (pd -> FEC_scheme));
}

	dataDescriptor::~dataDescriptor() {
}

